/* tslint:disable */

import { ITextEditorOptions } from '@vscode/monaco-editor/esm/vs/platform/editor/common/editor';
import { IEditor as ICodeEditor, IEditorViewState, ScrollType, IDiffEditor } from '@vscode/monaco-editor/esm/vs/editor/common/editorCommon';
import { isUndefinedOrNull } from '@vscode/monaco-editor/esm/vs/base/common/types';
import { IBaseResourceInput, IEditorOptions } from '@workbench-stack/core';
import { assign } from '@vscode/monaco-editor/esm/vs/base/common/objects';



// src/vs/workbench/browser/parts/editor/editor.ts

export const DEFAULT_EDITOR_PART_OPTIONS: IEditorPartOptions = {
	showTabs: true,
	highlightModifiedTabs: false,
	tabCloseButton: 'right',
	tabSizing: 'fit',
	focusRecentEditorAfterClose: true,
	showIcons: true,
	enablePreview: true,
	openPositioning: 'right',
	openSideBySideDirection: 'right',
	closeEmptyGroups: true,
	labelFormat: 'default',
	iconTheme: 'vs-seti'
};


export function getEditorPartOptions(config: IWorkbenchEditorConfiguration): IEditorPartOptions {
    const options: IEditorPartOptions = assign(Object.create(null), DEFAULT_EDITOR_PART_OPTIONS);

    if (!config || !config.workbench) {
        return options;
    }

    if (typeof config.workbench.iconTheme === 'string') {
        options.iconTheme = config.workbench.iconTheme;
    }

    if (config.workbench.editor) {
		assign(options, config.workbench.editor);
    }

    return options;
}


// src/vs/workbench/common/editor.ts

export interface IWorkbenchEditorConfiguration {
	workbench: {
		editor: IEditorPartConfiguration,
		iconTheme: string;
	};
}
interface IEditorPartConfiguration {
	showTabs?: boolean;
	highlightModifiedTabs?: boolean;
	tabCloseButton?: 'left' | 'right' | 'off';
	tabSizing?: 'fit' | 'shrink';
	focusRecentEditorAfterClose?: boolean;
	showIcons?: boolean;
	enablePreview?: boolean;
	enablePreviewFromQuickOpen?: boolean;
	closeOnFileDelete?: boolean;
	openPositioning?: 'left' | 'right' | 'first' | 'last';
	openSideBySideDirection?: 'right' | 'down';
	closeEmptyGroups?: boolean;
	revealIfOpen?: boolean;
	swipeToNavigate?: boolean;
	labelFormat?: 'default' | 'short' | 'medium' | 'long';
	restoreViewState?: boolean;
}

export interface IEditorPartOptions extends IEditorPartConfiguration {
	iconTheme?: string;
}


/**
 * The editor options is the base class of options that can be passed in when opening an editor.
 */
export class EditorOptions implements IEditorOptions {

	/**
	 * Helper to create EditorOptions inline.
	 */
	static create(settings: IEditorOptions): EditorOptions {
		const options = new EditorOptions();

		options.preserveFocus = settings.preserveFocus;
		options.forceReload = settings.forceReload;
		options.revealIfVisible = settings.revealIfVisible;
		options.revealIfOpened = settings.revealIfOpened;
		options.pinned = settings.pinned;
		options.index = settings.index;
		options.inactive = settings.inactive;
		options.ignoreError = settings.ignoreError;

		return options;
	}

	/**
	 * Tells the editor to not receive keyboard focus when the editor is being opened. By default,
	 * the editor will receive keyboard focus on open.
	 */
	preserveFocus: boolean | undefined;

	/**
	 * Tells the editor to reload the editor input in the editor even if it is identical to the one
	 * already showing. By default, the editor will not reload the input if it is identical to the
	 * one showing.
	 */
	forceReload: boolean | undefined;

	/**
	 * Will reveal the editor if it is already opened and visible in any of the opened editor groups.
	 */
	revealIfVisible: boolean | undefined;

	/**
	 * Will reveal the editor if it is already opened (even when not visible) in any of the opened editor groups.
	 */
	revealIfOpened: boolean | undefined;

	/**
	 * An editor that is pinned remains in the editor stack even when another editor is being opened.
	 * An editor that is not pinned will always get replaced by another editor that is not pinned.
	 */
	pinned: boolean | undefined;

	/**
	 * The index in the document stack where to insert the editor into when opening.
	 */
	index: number | undefined;

	/**
	 * An active editor that is opened will show its contents directly. Set to true to open an editor
	 * in the background.
	 */
	inactive: boolean | undefined;

	/**
	 * Will not show an error in case opening the editor fails and thus allows to show a custom error
	 * message as needed. By default, an error will be presented as notification if opening was not possible.
	 */
	ignoreError: boolean | undefined;
}

/**
 * Base Text Editor Options.
 */
export class TextEditorOptions extends EditorOptions {
    private startLineNumber: number;
    private startColumn: number;
    private endLineNumber: number;
    private endColumn: number;

    private revealInCenterIfOutsideViewport: boolean;
    private editorViewState: IEditorViewState | null;

    static from(input?: IBaseResourceInput): TextEditorOptions | undefined {
        if (!input || !input.options) {
            return undefined;
        }

        return TextEditorOptions.create(input.options);
    }

    /**
     * Helper to convert options bag to real class
     */
    static create(options: ITextEditorOptions = Object.create(null)): TextEditorOptions {
        const textEditorOptions = new TextEditorOptions();

        if (options.selection) {
            const selection = options.selection;
            textEditorOptions.selection(selection.startLineNumber, selection.startColumn, selection.endLineNumber, selection.endColumn);
        }

        if (options.viewState) {
            textEditorOptions.editorViewState = options.viewState as IEditorViewState;
        }

        if (options.forceReload) {
            textEditorOptions.forceReload = true;
        }

        if (options.revealIfVisible) {
            textEditorOptions.revealIfVisible = true;
        }

        if (options.revealIfOpened) {
            textEditorOptions.revealIfOpened = true;
        }

        if (options.preserveFocus) {
            textEditorOptions.preserveFocus = true;
        }

        if (options.revealInCenterIfOutsideViewport) {
            textEditorOptions.revealInCenterIfOutsideViewport = true;
        }

        if (options.pinned) {
            textEditorOptions.pinned = true;
        }

        if (options.inactive) {
            textEditorOptions.inactive = true;
        }

        if (options.ignoreError) {
            textEditorOptions.ignoreError = true;
        }

        if (typeof options.index === 'number') {
            textEditorOptions.index = options.index;
        }

        return textEditorOptions;
    }

    /**
     * Returns if this options object has objects defined for the editor.
     */
    hasOptionsDefined(): boolean {
        return !!this.editorViewState || (!isUndefinedOrNull(this.startLineNumber) && !isUndefinedOrNull(this.startColumn));
    }

    /**
     * Tells the editor to set show the given selection when the editor is being opened.
     */
    selection(startLineNumber: number, startColumn: number, endLineNumber: number = startLineNumber, endColumn: number = startColumn): EditorOptions {
        this.startLineNumber = startLineNumber;
        this.startColumn = startColumn;
        this.endLineNumber = endLineNumber;
        this.endColumn = endColumn;

        return this;
    }

    /**
     * Create a TextEditorOptions inline to be used when the editor is opening.
     */
    static fromEditor(editor: ICodeEditor, settings?: IEditorOptions): TextEditorOptions {
        const options = TextEditorOptions.create(settings);

        // View state
        options.editorViewState = editor.saveViewState();

        return options;
    }

    /**
     * Apply the view state or selection to the given editor.
     *
     * @return if something was applied
     */
    apply(editor: ICodeEditor, scrollType: ScrollType): boolean {

        // View state
        return this.applyViewState(editor, scrollType);
    }

    private applyViewState(editor: ICodeEditor, scrollType: ScrollType): boolean {
        let gotApplied = false;

        // First try viewstate
        if (this.editorViewState) {
            editor.restoreViewState(this.editorViewState);
            gotApplied = true;
        }

        // Otherwise check for selection
        else if (!isUndefinedOrNull(this.startLineNumber) && !isUndefinedOrNull(this.startColumn)) {

            // Select
            if (!isUndefinedOrNull(this.endLineNumber) && !isUndefinedOrNull(this.endColumn)) {
                const range = {
                    startLineNumber: this.startLineNumber,
                    startColumn: this.startColumn,
                    endLineNumber: this.endLineNumber,
                    endColumn: this.endColumn,
                };
                editor.setSelection(range);
                if (this.revealInCenterIfOutsideViewport) {
                    editor.revealRangeInCenterIfOutsideViewport(range, scrollType);
                } else {
                    editor.revealRangeInCenter(range, scrollType);
                }
            }

            // Reveal
            else {
                const pos = {
                    lineNumber: this.startLineNumber,
                    column: this.startColumn,
                };
                editor.setPosition(pos);
                if (this.revealInCenterIfOutsideViewport) {
                    editor.revealPositionInCenterIfOutsideViewport(pos, scrollType);
                } else {
                    editor.revealPositionInCenter(pos, scrollType);
                }
            }

            gotApplied = true;
        }

        return gotApplied;
    }
}
