import { IConfigurationOverrides } from '@workbench-stack/core';
import { URI } from '@vscode/monaco-editor/esm/vs/base/common/uri';
import * as objects from '@vscode/monaco-editor/esm/vs/base/common/objects';
import * as arrays from '@vscode/monaco-editor/esm/vs/base/common/arrays';
import * as types from '@vscode/monaco-editor/esm/vs/base/common/types';

export function getConsolidatedConfigurationModelForResource({ resource }: IConfigurationOverrides, workspace: Workspace | undefined): ConfigurationModel {
    let consolidateConfiguration = getWorkspaceConsolidatedConfiguration();

    if (workspace && resource) {
        const root = workspace.getFolder(resource);
        if (root) {
            consolidateConfiguration = getFolderConsolidatedConfiguration(root.uri) || consolidateConfiguration;
        }
        const memoryConfigurationForResource = _memoryConfigurationByResource.get(resource);
        if (memoryConfigurationForResource) {
            consolidateConfiguration = consolidateConfiguration.merge(memoryConfigurationForResource);
        }
    }

    return consolidateConfiguration;
}

 export function getConsolidateConfigurationModel(overrides: IConfigurationOverrides, workspace: Workspace | undefined): ConfigurationModel {
    let configurationModel = getConsolidatedConfigurationModelForResource(overrides, workspace);
    return overrides.overrideIdentifier ? configurationModel.override(overrides.overrideIdentifier) : configurationModel;
}

export function getFolderConsolidatedConfiguration(folder: URI): ConfigurationModel {
    let folderConsolidatedConfiguration = _foldersConsolidatedConfigurations.get(folder);
    if (!folderConsolidatedConfiguration) {
        const workspaceConsolidateConfiguration = getWorkspaceConsolidatedConfiguration();
        const folderConfiguration = _folderConfigurations.get(folder);
        if (folderConfiguration) {
            folderConsolidatedConfiguration = workspaceConsolidateConfiguration.merge(folderConfiguration);
            if (_freeze) {
                folderConsolidatedConfiguration = folderConsolidatedConfiguration.freeze();
            }
            _foldersConsolidatedConfigurations.set(folder, folderConsolidatedConfiguration);
        } else {
            folderConsolidatedConfiguration = workspaceConsolidateConfiguration;
        }
    }
    return folderConsolidatedConfiguration;
}


function getWorkspaceConsolidatedConfiguration(): ConfigurationModel {
    if (!_workspaceConsolidatedConfiguration) {
        _workspaceConsolidatedConfiguration = _defaultConfiguration.merge(userConfiguration, _workspaceConfiguration, _memoryConfiguration);
        if (_freeze) {
            _workspaceConfiguration = _workspaceConfiguration.freeze();
        }
    }
    return _workspaceConsolidatedConfiguration;
}

function merge(...others: ConfigurationModel[]): ConfigurationModel {
    const contents = objects.deepClone(contents);
    const overrides = objects.deepClone(overrides);
    const keys = [...keys];

    for (const other of others) {
        mergeContents(contents, other.contents);

        for (const otherOverride of other.overrides) {
            const [override] = overrides.filter(o => arrays.equals(o.identifiers, otherOverride.identifiers));
            if (override) {
                mergeContents(override.contents, otherOverride.contents);
            } else {
                overrides.push(objects.deepClone(otherOverride));
            }
        }
        for (const key of other.keys) {
            if (keys.indexOf(key) === -1) {
                keys.push(key);
            }
        }
    }
    return new ConfigurationModel(contents, keys, overrides);
}


function mergeContents(source: any, target: any): void {
    for (const key of Object.keys(target)) {
        if (key in source) {
            if (types.isObject(source[key]) && types.isObject(target[key])) {
                mergeContents(source[key], target[key]);
                continue;
            }
        }
        source[key] = objects.deepClone(target[key]);
    }
}
