
// tslint:disable
import { localize } from '@vscode/monaco-editor/esm/vs/nls';
import {
    ILabelService, ClientTypes, IEnvironmentService, IClientWorkspaceContextService as IWorkspaceContextService,
    IWorkspaceIdentifier, ISingleFolderWorkspaceIdentifier, isSingleFolderWorkspaceIdentifier, WORKSPACE_EXTENSION, toWorkspaceIdentifier, isWorkspaceIdentifier, IWorkspace,
} from '@workbench-stack/core';
import { tildify, getPathLabel } from '@vscode/monaco-editor/esm/vs/base/common/labels';
import { inject, injectable } from 'inversify';
import { URI } from '@vscode/monaco-editor/esm/vs/base/common/uri';
import { match } from '@vscode/monaco-editor/esm/vs/base/common/glob';
import { Schemas } from '@vscode/monaco-editor/esm/vs/base/common/network';
import { isEqual, basenameOrAuthority, isEqualOrParent, basename, joinPath, dirname } from '@vscode/monaco-editor/esm/vs/base/common/resources';
// import { isWindows } from 'vs/base/common/platform';
import { ltrim, endsWith } from '@vscode/monaco-editor/esm/vs/base/common/strings';
import { ResourceLabelFormatter, ResourceLabelFormatting } from '@vscode/monaco-editor/esm/vs/platform/label/common/label';
// import { ExtensionsRegistry } from 'vs/workbench/services/extensions/common/extensionsRegistry';
// import { LifecyclePhase } from '@vscode/monaco-editor/esm/vs/platform/lifecycle/common/lifecycle';
// import { registerSingleton } from 'vs/platform/instantiation/common/extensions';

@injectable()
export class LabelService implements ILabelService {

    private formatters: ResourceLabelFormatter[] = [];

    constructor(
        @inject(ClientTypes.IEnvironmentService) private readonly environmentService: IEnvironmentService,
        @inject(ClientTypes.IWorkspaceContextService) private readonly contextService: IWorkspaceContextService,
    ) {

    }

    public findFormatting(resource: URI): ResourceLabelFormatting | undefined {
        let bestResult: ResourceLabelFormatter | undefined;

        this.formatters.forEach(formatter => {
            if (formatter.scheme === resource.scheme) {
                if (!bestResult && !formatter.authority) {
                    bestResult = formatter;
                    return;
                }
                if (!formatter.authority) {
                    return;
                }

                // tslint:disable-next-line:max-line-length
                if (match(formatter.authority, resource.authority) && (!bestResult || !bestResult.authority || formatter.authority.length > bestResult.authority.length || ((formatter.authority.length === bestResult.authority.length) && formatter.priority))) {
                    bestResult = formatter;
                }
            }
        });

        return bestResult ? bestResult.formatting : undefined;
    }

    public getUriLabel(resource: URI, options: { relative?: boolean, noPrefix?: boolean, endWithSeparator?: boolean } = {}): string {
        const formatting = this.findFormatting(resource);
        if (!formatting) {
            return getPathLabel(resource.path, this.environmentService, options.relative ? this.contextService : undefined);
        }

        let label: string | undefined;
        const baseResource = this.contextService && this.contextService.getWorkspaceFolder(resource);

        if (options.relative && baseResource) {
            let relativeLabel: string;
            if (isEqual(baseResource.uri, resource)) {
                relativeLabel = ''; // no label if resources are identical
            } else {
                const baseResourceLabel = this.formatUri(baseResource.uri, formatting, options.noPrefix);
                relativeLabel = ltrim(this.formatUri(resource, formatting, options.noPrefix).substring(baseResourceLabel.length), formatting.separator);
            }

            const hasMultipleRoots = this.contextService.getWorkspace().folders.length > 1;
            if (hasMultipleRoots && !options.noPrefix) {
                const rootName = (baseResource && baseResource.name) ? baseResource.name : basenameOrAuthority(baseResource.uri);
                relativeLabel = relativeLabel ? (rootName + ' • ' + relativeLabel) : rootName; // always show root basename if there are multiple
            }

            label = relativeLabel;
        } else {
            label = this.formatUri(resource, formatting, options.noPrefix);
        }

        return options.endWithSeparator ? this.appendSeparatorIfMissing(label, formatting) : label;
    }

    getWorkspaceLabel(workspace: (IWorkspaceIdentifier | ISingleFolderWorkspaceIdentifier | IWorkspace), options?: { verbose: boolean }): string {
        if (IWorkspace.isIWorkspace(workspace)) {
            const identifier = toWorkspaceIdentifier(workspace);
            if (!identifier) {
                return '';
            }

            workspace = identifier;
        }

        // Workspace: Single Folder
        if (isSingleFolderWorkspaceIdentifier(workspace)) {
            // Folder on disk
            const label = options && options.verbose ? this.getUriLabel(workspace) : basename(workspace) || '/';
            return this.appendWorkspaceSuffix(label, workspace);
        }

        if (isWorkspaceIdentifier(workspace)) {
            // Workspace: Untitled
            if (isEqualOrParent(workspace.configPath, this.environmentService.untitledWorkspacesHome)) {
                return localize('untitledWorkspace', "Untitled (Workspace)");
            }

            // Workspace: Saved
            let filename = basename(workspace.configPath);
            if (endsWith(filename, WORKSPACE_EXTENSION)) {
                filename = filename.substr(0, filename.length - WORKSPACE_EXTENSION.length - 1);
            }
            let label;
            if (options && options.verbose) {
                label = localize('workspaceNameVerbose', "{0} (Workspace)", this.getUriLabel(joinPath(dirname(workspace.configPath), filename)));
            } else {
                label = localize('workspaceName', "{0} (Workspace)", filename);
            }
            return this.appendWorkspaceSuffix(label, workspace.configPath);
        }
        return '';

    }
    public getSeparator(scheme: string, authority?: string): '/' | '\\' {
        const formatter = this.findFormatting(URI.from({ scheme, authority }));
        return formatter && formatter.separator || '/';
    }

    public getHostLabel(scheme: string, authority?: string): string {
        const formatter = this.findFormatting(URI.from({ scheme, authority }));
        return formatter && formatter.workspaceSuffix || '';
    }


    private formatUri(resource: URI, formatting: ResourceLabelFormatting, forceNoTildify?: boolean): string {
        let label = formatting.label.replace(labelMatchingRegexp, match => {
            switch (match) {
                case '${scheme}': return resource.scheme;
                case '${authority}': return resource.authority;
                case '${path}': return resource.path;
                default: return '';
            }
        });

        // convert \c:\something => C:\something
        // if (formatting.normalizeDriveLetter && hasDriveLetter(label)) {
        //     label = label.charAt(1).toUpperCase() + label.substr(2);
        // }

        if (formatting.tildify && !forceNoTildify) {
            label = tildify(label, this.environmentService.userHome);
        }
        if (formatting.authorityPrefix && resource.authority) {
            label = formatting.authorityPrefix + label;
        }

        return label.replace(sepRegexp, formatting.separator);
    }

    private appendSeparatorIfMissing(label: string, formatting: ResourceLabelFormatting): string {
        let appendedLabel = label;
        if (!endsWith(label, formatting.separator)) {
            appendedLabel += formatting.separator;
        }
        return appendedLabel;
    }

    private appendWorkspaceSuffix(label: string, uri: URI): string {
        if (uri.scheme === Schemas.file) {
            return label;
        }

        const formatting = this.findFormatting(uri);
        const suffix = formatting && (typeof formatting.workspaceSuffix === 'string') ? formatting.workspaceSuffix : uri.scheme;
        return suffix ? `${label} [${suffix}]` : label;
    }
}


const sepRegexp = /\//g;
const labelMatchingRegexp = /\$\{scheme\}|\$\{authority\}|\$\{path\}/g;
// function hasDriveLetter(path: string): boolean {
//     return !!(isWindows && path && path[2] === ':');
// }
